/**
 * Create an ebay assignment controller that handles the display of ebay items.
 *
 * The `.ebayAssignmentcontroller()` method creates an ebay assignment controller.
 *
 * ### Examples
 * create an ebay assignment controller with gtin inputfield and searchbutton.
 *
 * require([
 *     "jquery",
 *     "ep",
 *
 *     "de_epages/ebay/ui/ebayassignmentcontroller",
 *     "$ready!"
 * ], function($, ep) {
 *     var assignmentController = $('.ep-ebay-assignment-controller').ebayAssignmentcontroller({
 *       epid: 219540209,
 *       searchButton: '.ep-ebay-assignment-search-button',
 *       searchInput:  '.ep-ebay-assignment-search-input',
 *       objectId: '#ID'
 *   })
 *
 * });
 *
 * HTML:
 *
 * <div class="col-xs-12 BottomSmallPadding NoWrap">
 *     <input class="ep-js ep-ebay-assignment-search-input" data-js=".uiInput()" type="text" name="UPCEAN"  value=""/>
 *     <button class="ep-ebay-assignment-search-button" type="button" title="{Search}">{Search}</button>
 * </div>
 * <div class="ep-ebay-assignment-controller"></div>
 *
 * @class de_epages.ebay.ui.ebayAssignmentcontroller
 * @extends jQuery.widget
 *
 * @uses jQuery.dict
 * @uses jQuery.tmpl
 * @uses jQuery.i18n
 * @uses jQuery.ui.widget
 * @uses de_epages
 * @since 7.26.0
 */

/**
 * @cfg {Number} [epid] An ebay product id (optional).
 */

/**
 * @cfg {String} [searchButton] Valid jquery selector representing a search button.
 */

/**
 * @cfg {String} [searchInput] Valid jquery selector representing an input field.
 */

/**
 * @cfg {String} [saveButton] Valid jquery selector representing the save button of the form.
 */

/**
 * @cfg {String} [errorElem] Valid jquery selector representing an element containing the error messages.
 */

/**
 * @cfg {String} [objectId] Object Id for the ebay auction object.
 */

/**
 * See `jQuery.ui.ebayAssignmentcontroller` for details.
 *
 * @param {Number} [epid] An ebay product id (optional).
 * @param {String} [searchButton] Valid jquery selector representing a search button.
 * @param {String} [searchInput] Valid jquery selector representing an input field.
 * @param {String} [saveButton] Valid jquery selector representing the save button of the form.
 * @param {String} [errorElem] Valid jquery selector representing an element containing the error messages.
 * @param {String} [objectId] Object Id for the ebay auction object.
 *
 * @method ebayAssignmentcontroller
 * @member jQuery
 *
 * @since 7.26.0
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.ebay.ui.ebayassignmentcontroller
 */

define("de_epages/ebay/ui/ebayassignmentcontroller", [
    "jquery",
    "ep",
    "de_epages",
    "ep/ui/simple-tooltip",
    "$dict!../dictionary",
    "$tmpl!./ebayassignmentcontroller",
    "$tmpl!./ebayassignmentcontrollererror",

    "ep/validate",
    "ep/ajax",
    "jquery/ui/widget",
    "ep/fn/busy",
    "jquery/tmpl"
], function($, ep, de_epages, tooltip, ebayDictionary, tmplEbayAssignmentController, tmplEbayAssignmentControllerError) {
    "use strict";

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Apply}
     *                  {Error}
     *                  {INVALID_LENGTH}
     *                  {MsgInvalidProductCode}
     *                  {UnknownError}
     *
     * @dictionary      de_epages.ebay.dictionary
     *
     * @translation     {noShopEnvironment}
     *                  {EbayRequestFailed}
     *                  {EbayItems}
     *                  {EbayAttributes}
     *                  {NoEbayItemsFound}
     *                  {UnityConnectionNotSaved}
     *                  {PleaseConsultYourProvider}
     */

    // translation object
    var translations = {
            EbayWarningNoEbayEnvironment: ebayDictionary.translate('noShopEnvironment'),
            NoEbayEnvironment: ebayDictionary.translate('noShopEnvironment'),
            EbayItems: ebayDictionary.translate('EbayItems'),
            EbayAttributes: ebayDictionary.translate('EbayAttributes'),
            NoEbayItemsFound: ebayDictionary.translate('NoEbayItemsFound'),
            UnityConnectionNotSaved: ebayDictionary.translate('UnityConnectionNotSaved'),
            PleaseConsultYourProvider: ebayDictionary.translate('PleaseConsultYourProvider'),

            error: ebayDictionary.translate('Error'),
            unknownError: ebayDictionary.translate('UnknownError'),
            apply: ebayDictionary.translate('Apply'),
            INVALID_LENGTH: ebayDictionary.translate('INVALID_LENGTH'),
            MsgInvalidProductCode: ebayDictionary.translate('MsgInvalidProductCode')
        },
        errors = {
            "ApiCallError": {
                header: translations.error,
                description: translations.UnityConnectionNotSaved + " " + translations.PleaseConsultYourProvider
            },
            "DefaultError": {
                header: translations.error,
                description: translations.unknownError
            }
        };

    $.widget('ui.ebayAssignmentcontroller', {
    // dom,
    // searchButton,
    // searchInput,
    // saveButton,
    // errorElem,
    // busyElem,
    // lastValidGtin,

        /**
         * Options to be passed to the widgt on initialzation
         *
         * @cfg {Object} options
         * @cfg {Object} options.controller             controller object for rendering the categories
         */
        options: {
        // epid,
        // searchButton,
        // searchInput,
        // saveButton,
        // errorElem,
        // busyElem,
        // objectId
        },

        /**
         * Magic jquery function.
         *
         * @method _create
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _create: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                value;

            // get search button element
            self._getElement('searchButton');
            // get save button element
            self._getElement('saveButton');
            // get element for error messages
            self._getElement('errorElem');
            // get search input element
            self._getElement('searchInput');
            // get busy element
            self._getElement('busyElem');

            // if the search field exists
            if (self.searchInput) {
                value = self.searchInput.val();

                // if the search field contains a value
                if (value.length) {
                    // if containing value is a valid ean/upc or isbn
                    if (self._validate(value)) {
                        // search for value
                        self._search(value);
                    }
                } else {
                    // reset to empty if no search value exists
                    self._resetResult({
                        empty: true
                    });
                }

                // register event handler for search input
                self.searchInput.on('input', function() {
                    // set save button disabled on input (except: empty field)
                    self.disableSaveButton((this.value !== "") ? true : false);
                });
            }

            // if a search button and a search input exist => register click event for search button
            if (self.searchButton && self.searchInput) {
                self.searchButton.on('click.ebayAssignmentcontroller', function(evt) {
                    evt.preventDefault();

                    var value = self.searchInput.val();

                    // validate search value
                    if (!self._validate(value)) {
                        return false;
                    }

                    // if the search field contains a value
                    if (value.length) {
                        // search for value
                        self._search(value);
                    } else {
                        // reset to empty if no search value exists
                        self._resetResult({
                            empty: true
                        });
                    }
                });
            }
        },

        /**
         * This method renders the tabĺe with the search results.
         *
         * @method _renderSearchTable
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _renderSearchTable: function(options) {
            var self = this,
                o = self.options,
                elem = self.element,
                renderedTmpl;

            // render view template for ebay categories
            renderedTmpl = tmplEbayAssignmentController(
                [{
                    tEbayItems: translations.EbayItems,
                    tEbayAttributes: translations.EbayAttributes,
                    tNoEbayItemsFound: translations.NoEbayItemsFound,
                    products: options.products,
                    currentEpid: o.epid
                }]
            );

            // if epid radio buttons exist => remove event handler
            if (self.dom && self.dom.epidRadio) {
                self.dom.epidRadio.off('change.ebayAssignmentcontroller');
                self.dom.epidRadio.off('click.ebayAssignmentcontroller');
            }

            // initialize object for category DOM elements
            self.dom = {};

            // load template and push named tmpl elements to current instance
            $.extend(self.dom, renderedTmpl.tmplItem('elements'));
            // remove content
            elem.empty();
            // append base template
            elem.append(renderedTmpl);

            // if radio button for epid exists
            if (self.dom.epidRadio) {
                // register events
                // change event
                self.dom.epidRadio.on('change.ebayAssignmentcontroller', function() {
                    var $this = $(this);

                    if ($this.prop('checked')) {
                        o.epid = $this.val();
                        // set last valid GTIN in search input
                        if (self.searchInput && self.lastValidGtin) {
                            self.searchInput.val(self.lastValidGtin);
                        }
                        // enalbled save button
                        self.disableSaveButton(false);
                    }
                });
                // click event
                self.dom.epidRadio.on('click.ebayAssignmentcontroller', function() {
                    // enable/disable save button depending on the number of checked radio buttons
                    self.disableSaveButton(!self.dom.epidRadio.filter(':checked').length);
                });

                // enable/disable save button depending on the number of checked radio buttons
                self.disableSaveButton(!self.dom.epidRadio.filter(':checked').length);
            }
        },

        /**
         * This method search for an ebay product.
         *
         * @method _search
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
         _search: function(value) {
            var self = this,
                o = self.options;

            self.disableSaveButton(true);
            self.busyElem && self.busyElem.busy('true');

            // initiate ajax request to get results from the ebay API
            ep.ajax({
                dataType: "json",
                data: {
                    'ChangeAction': 'JSONSearchCatalogProductByGtin',
                    'ViewAction': 'JSONViewResponse',
                    'SearchString': value,
                    'ObjectID': o.objectId
                },
                type: "post"
            })
            .done(function (data) {
                if (data.success !== undefined) {
                    // if request was successful => render table and hide error element
                    if (data.success) {
                        self._renderSearchTable({
                            products: JSON.parse(data.products)
                        });
                        // save last valid GTIN
                        self.lastValidGtin = value;
                        // hide error element
                        self.errorElem && self.errorElem.addClass("HideElement");
                    } else {
                        // error handling
                        data.error && self._renderError(data.error);
                    }
                }
                self.busyElem && self.busyElem.busy('hide');
            })
            .fail(function () {
                // console.log('fail: ', arguments);
                self._renderError('DefaultError');
                self.busyElem && self.busyElem.busy('hide');
            });
        },

         /**
         * This method is for validation of ean and isbn.
         *
         * @method _validate
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _validate: function(value) {
            var self = this;

            // if number of characters are not inside the allowed range => show tooltip and reset results
            if ((value.length > 0) && ((value.length < 8) || (value.length > 13))) {
                tooltip.show({
                    element: self.searchInput,
                    interactiveElement: $('body'),
                    eventtype: 'mousemove',
                    message: translations.INVALID_LENGTH,
                    orientation: 'bottom'
                });

                self._resetResult();

                return false;
            }

            /* EAN/UPC and ISBN validation */
            // if value is neither a valid ean/gtin nor a valid isbn => show tooltip and reset results
            if ((value.length > 0) && ((ep.validate.gtin(value) !== true) && (ep.validate.isbn10(value) !== true) && (ep.validate.isbn13(value) !== true))) {
                tooltip.show({
                    element: self.searchInput,
                    interactiveElement: $('body'),
                    eventtype: 'mousemove',
                    message: translations.MsgInvalidProductCode,
                    orientation: 'bottom'
                });

                self._resetResult();

                return false;
            }

            return true;
        },

        /**
         * This method handles the error and renders error messages.
         *
         * @method _renderError
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _renderError: function(err) {
            var self = this,
                tmpError = errors[err] ? errors[err] : errors.DefaultError,
                renderedTmpl;

            if (self.errorElem) {
                // render error template
                renderedTmpl = tmplEbayAssignmentControllerError(
                    [{
                        tErrorHeader: tmpError.header,
                        tErrorDescription: tmpError.description
                    }]
                );

                self.errorElem.empty();
                self.errorElem.append(renderedTmpl);
                self.errorElem.removeClass("HideElement");
            }

        },

         /**
         * This method resets the result table and disabled the button and remove the current epid.
         *
         * @method _resetResult
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _resetResult: function(opts) {
            var self = this,
                o = self.options,
                empty = (opts && opts.empty) ? opts.empty : false;

            // render result table without results and delete epid property
            self._renderSearchTable({
                products: []
            });
            o.epid = undefined;

            // reset last valid GTIN
            if (self.lastValidGtin && empty) {
                self.lastValidGtin = '';
            }

            // disabled save button
            self.disableSaveButton(!empty);
        },

         /**
         * This public method disables or enables the save button.
         *
         * @method disableSaveButton
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        disableSaveButton: function(disable) {
            var self = this;

            // disable/enable save button
            self.saveButton && self.saveButton.prop('disabled', disable);
        },

        /**
         * This method checks if an element is a jQuery object (returns true/false).
         *
         * @method _isJQueryElement
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _isJQueryElement: function(element) {
            return (element && (typeof element === "object" && element[0] && element[0].tagName));
        },

        /**
         * This method builds a widget element containing a jquery representation of a dom node or undefined.
         *
         * @method _getElement
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        _getElement: function(prop) {
            var self = this,
                o = self.options;

            if (o[prop]) {
                // get search button
                self[prop] = $(o[prop]);

                // if search filed is not a valid jquery element => delete again
                if (!self._isJQueryElement(self[prop])) {
                    self[prop] = undefined;
                }
            }
        },

        /**
         * This method removes the container on which the Uploader has been appended to.
         *
         * @method destroy
         * @member de_epages.ebay.ui.ebayassignmentcontroller
         *
         * @since 7.26.0
         */
        destroy: function() {
            var self = this;

            self._superApply(arguments);

            if (self.searchButton) {
                self.searchButton.off('click.ebayAssignmentcontroller');
            }

            if (self.dom.epidRadio) {
                self.dom.epidRadio.off('change.ebayAssignmentcontroller');
            }

            self.element.empty();
        }

    });

    return de_epages;
});
